# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (c) 2010 Luigi Grilli (luigi.grilli at gmail.com)
#                          All Rights Reserved.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


import pooler
import time
import datetime
import netsvc
import base64
import tools
import threading
from osv import fields, osv
from tools.translate import _
from tempfile import TemporaryFile
from tools.misc import debug

# Constants

# Tempo di consegna materiale espresso in giorni lavorativi
# come da standard Manuale Listino Rel. 1.1 - rev.10 - Marzo 2011
# http://www.metel.it/stc/DownloadPubFile.pub_do/(id)402882a82ed6bcdd012ed77d44190002/Manuale_Listino_Rel_1.1_rev_10_Marzo_2011.zip
LEADTIME_CONVERSION_TABLE = {
        
        # I numeri esprimono direttamente i giorni
        # lavorativi previsti per la consegna
        '1': 1,
        '2': 2,
        '3': 3,
        '4': 4,
        '5': 5,
        '6': 6,
        '7': 7,
        '8': 8,
        '9': 9,
        
        # Le lettere esprimono le settimane previste per la consegna,
        # ogni settimana comprende 5 giorni lavorativi e si parte da
        # due settimane (quindi A = 2 sett = 10 giorni lavorativi)
        'A': 2 * 5,
        'B': 3 * 5,
        'C': 4 * 5,
        'D': 5 * 5,
        'E': 6 * 5,
        'F': 7 * 5,
        'G': 8 * 5,
        'H': 9 * 5,
        'I': 10 * 5,
        'J': 11 * 5,
        'K': 12 * 5,
        'L': 13 * 5,
        'M': 14 * 5,
        'N': 15 * 5,
        'O': 16 * 5,
        'P': 17 * 5,
        'Q': 18 * 5,
        'R': 19 * 5,
        'S': 20 * 5,
        'T': 21 * 5,
        'U': 22 * 5,
        'V': 23 * 5,
        'W': 24 * 5,
        'X': 25 * 5,
        'Y': 26 * 5,
        'Z': 27 * 5,
                              }

class MetelImport(threading.Thread):
    def Init(self, cr, uid, data):
        self.uid = uid
        
        self.dbname = cr.dbname
        self.pool = pooler.get_pool(cr.dbname)
        self.cr = cr
        
        self.uo_new = 0
        self.uo_update = 0
        self.SetFile(data)
        
        return self.ReadMetelHeader()
    
    def SetFile(self, data):
        self.fileobj = TemporaryFile()
        self.fileobj.write( data )
        self.fileobj.seek(0)
        return
    
    def run(self):
        self.cr = pooler.get_db(self.dbname).cursor()
        self.pool = pooler.get_pool(self.cr.dbname)

        self.ImportFile()
        
        body = _("Imported products: %d \nUpdated products: %d") % (self.uo_new, self.uo_update)
        request = self.pool.get('res.request')
        request.create(self.cr, self.uid, {
            'name': _("Metel Import Completed"),
            'act_from': self.uid,
            'act_to': self.uid,
            'ref_partner_id': self.partner_id,
            'state': 'waiting',
            'body': body,
            'active': True,
        })
        
        self.cr.commit()
        self.cr.close()
        
        return
    
    def ReadMetelHeader(self):
        line = self.fileobj.readline()
        self.metel = line[0:20].strip()
        self.prod = line[20:23]
        self.desc = line[56:125].strip()
        self.date_start = datetime.date(int(line[40:44]), int(line[44:46]), int(line[46:48]))
        self.last_change = datetime.date(int(line[48:52]), int(line[52:54]), int(line[54:56]))
        
        # is there a matching partner?
        partner_ids = self.pool.get('res.partner').search(self.cr, self.uid, [('metel_code', '=', self.prod)])
        
        if partner_ids:
            self.partner_id = partner_ids[0]
            #get partner record
            self.partner_obj = self.pool.get('res.partner').browse(self.cr, self.uid, partner_ids)
            if self.partner_obj[0].metel_category:
                self.cat_id = self.partner_obj[0].metel_category.id
                return 1
            
            return -2
        else:
            self.partner_id = 0
            self.partner_obj = None
            self.cat_id = 0
        
        return -1
    
    def GetDiscountFamily(self, name):
        '''Get the category that match the specified discount family, in not found create it
        
        Args:
          name: Discount family name
          
        Returns:
          The category ID of the product
        '''
        if len(name) > 0:
            #is there a matching category ?
            family_id = self.pool.get('product.category').search(self.cr, self.uid, [('name', '=', name), ("parent_id", '=', self.cat_id)])
            
            # family found
            if family_id:
                return family_id[0]
            #family discount not found, create it
            else:
                return self.pool.get('product.category').create(self.cr, self.uid,
                {
                    'name': name,
                    'parent_id': self.cat_id,
                })
        
        #if category name is empty return the main import category ID
        return self.cat_id

    def GetUOM(self, uom):
        '''Get the unity of measure specified, in not found create it
        
        Args:
          name: UOM name
          
        Returns:
          The UOM ID of the product
        '''
        
        if len(uom) <= 0:
            return 0
        
        #is there a matching category ?
        uom_id = self.pool.get('product.uom').search(self.cr, self.uid, [('name', '=', uom)])
        
        if uom_id:
            return uom_id[0]
        else:
            uom_cat_ids = self.pool.get('product.uom.categ').search(self.cr, self.uid, [('name', '=', 'Unit')])
            return self.pool.get('product.uom').create(self.cr, self.uid,
            {
             'name': uom,
             'category_id': uom_cat_ids[0],
            })

        return 0
        
    def ImportFile(self):
        if self.partner_id > 0:
            #Import Products
            for line in self.fileobj:
                self.ImportRow(line)
        return
        
    def ImportRow(self, line):
        prod = line[0:3]
        code = line[3:19].strip()
        ean13 = line[19:32]
        desc = line[32:75].strip()
        qmin = int(line[85:90].strip())
        
        # Conversione leadtime espresso con codice Metel
        # in giorni lavorativi
        leadtimeCode = line[96:97].upper()
        leadtime = LEADTIME_CONVERSION_TABLE[leadtimeCode]
        
        price1 = float(line[97:108].strip()) / 100
        price2 = float(line[108:119].strip()) / 100
        price_multi = float(line[119:125].strip());
        price1 /= price_multi
        price2 /= price_multi
        uom = line[128:131].strip()
        family_discount = line[141:159].strip()
        family_stat = line[159:177].strip()
        
        if len(code) <= 0:
            return
        
        print "code:", code, " partner_id:", self.partner_id
        # is there a matching product ?
        product_id = self.pool.get('product.product').search(self.cr, self.uid, [('manufacturer', '=', self.partner_id), ("manufacturer_pref", '=', code)])
        
        # product found, update it
        if product_id:
            self.uo_update += 1
            product = self.pool.get('product.product').write(self.cr, self.uid, product_id[0], {'standard_price': price1})
            
            # is there a matching supplier info ?
            supplier_id = self.pool.get('product.supplierinfo').search(self.cr, self.uid, [('product_id', '=', product_id[0])])
            
            #updated supplier info with leadtime and minimun quantity
            if supplier_id:
                supplier = self.pool.get('product.supplierinfo').write(self.cr, self.uid, supplier_id[0], {'qty': qmin, 'delay': leadtime})
            # create the supplier info for the manufacturer
            else:
                # TODO: Passare da product_id a product_template_id
                supplier = self.pool.get('product.supplierinfo').create(self.cr, self.uid,
                {
                 'product_id': product_id[0],
                 # TODO: controllare cosa fare con qty e min_qty
                 'qty': qmin,
                 'min_qty': qmin,
                 'delay': leadtime,
                 'name': self.partner_id,
                 'product_code': code,
                 'product_name': desc,
                })
            
        #product not found, create it. we use the code and the description of the manufacturer
        else:
            self.uo_new += 1
            
            product_cat = self.GetDiscountFamily(family_discount)
            product_uom = self.GetUOM(uom)
            
            product = self.pool.get('product.product').create(self.cr, self.uid,
            {
             'default_code': code,
             'name': desc,
             'ean13': ean13,
             'manufacturer': self.partner_id,
             'manufacturer_pref': code,
             'manufacturer_pname': desc,
             'standard_price': price1,
             'list_price': price2,
             'categ_id': product_cat,
             'uom_id': product_uom,
             'uom_po_id': product_uom,
            })
            
            # Commit changes to DB before creating new supplier info
            self.cr.commit()
            
            # Get the product_tempalte ID
            productObject = self.pool.get('product.product').browse(self.cr, self.uid, product)
            product_template_id = productObject.product_tmpl_id.id
            
            # create the supplier info for the manufacturer
            supplier = self.pool.get('product.supplierinfo').create(self.cr, self.uid,
            {
             'product_id': product_template_id,
             # TODO: controllare cosa fare con qty e min_qty
             'qty': qmin,
             'min_qty': qmin,
             'delay': leadtime,
             'name': self.partner_id,
             'product_code': code,
             'product_name': desc,
            })
